/*** File: Node.java
*** Contains: Class defining the activity node element of CPE.
*** Author: Steve Polyak <Steve_Polyak@ed.ac.uk>
*** Created: June 1997
*** Updated: Wed Jan 21 19:09:38 1998 by Jeff Dalton
*/

package ix.acp3;

import java.util.*;
import java.awt.*;

class Node {
  int m_x;
  int m_y;
  String m_lbl;
  String m_key;
  
  boolean shadow = false;       // *** added by JL ***
  String expansion = "";        // *** added by JL ***
  boolean dummyNode = false;    // *** added by JL ***

  // Flags for node states.
  // BeginFocus = user passed over node while
  //   in "select begin node" mode
  // BeginSelected = user clicked on the node
  //   as the "begin node"
  boolean m_selected = false;
  boolean m_beginSelected = false;
  boolean m_beginFocus = false;
  boolean m_endFocus = false;
  // status:
  // 0 = not yet
  // 1 = complete
  // 2 = executing
  // 3 = possible 
  // 4 = impossible
  int m_status = 0;
  
  // Graphics properties
  final Color m_nodeColor = Color.white;
  final Color m_selectedColor = Color.lightGray;
  final Color m_beginSelectedColor = Color.yellow;
  final Color m_endSelectedColor = Color.yellow;
  protected  Color  m_statusColors[] = { new Color(0xFFFFFF),  //white
					 new Color(0x99FFFF),  //blue
					 new Color(0x99FF99),  //green
					 new Color(0xFFCC66),  //orange
					 new Color(0xFF6699),  //red
					 new Color(0xCCCCCC),  //light gray
					 new Color(0x999999)}; //dark gray

//protected  Color  m_statusColors[] =  { new Color(0xFFFFFF),
//                                        new Color(0xbbf7ff),
//                                        new Color(0xceffce),
//                                        new Color(0xf7e3ab),
//                                        new Color(0xffbfb8) };
  Rectangle m_rect;
  
  Node(String key,int x, int y) {
    m_key = key;
    m_lbl = key;
    m_rect = new Rectangle(x, y, 0, 0);
    m_x = x;
    m_y = y;
  }
  
  public void setStatus(int status) {
    m_status = status;
  }
  
  // *** added by JL ***
  public void setExpanded(boolean flag, String expansion) {
    this.shadow = flag;
    this.expansion = expansion;
  }

  public void draw(Graphics g, FontMetrics fm) {
    int x = m_x;
    int y = m_y;
    
    String text = ((dummyNode ? "" : m_lbl));

    int w = fm.stringWidth(text) + 10;
    int h = fm.getHeight() + 4;
    m_rect.width = w;
    m_rect.height = h;
    m_rect.x = x - w/2;
    m_rect.y = y - h / 2;

    if (dummyNode) {
      return;
    }

    //if has expansion, draw a shadow
    if (shadow) {
      g.setColor(Color.darkGray);
      drawShadow(g,(x - w/2)+3, (y - m_rect.height / 2)+3, w,
		 m_rect.height);
    }

    if (m_selected)
      g.setColor(m_selectedColor);
    else if (m_beginSelected)
      g.setColor(m_beginSelectedColor);
    else
      g.setColor(m_statusColors[m_status]);
    
    g.fillRect(x - w/2, y - h / 2, w, h);
    
    if(m_beginFocus) 
      g.setColor(m_beginSelectedColor);
    else if (m_endFocus)
      g.setColor(m_endSelectedColor);
    else
      g.setColor(Color.black);
    
    g.drawRect(x - w/2, y - h / 2, w-1, h-1);
    g.drawString(text, x - (w-10)/2, (y - (h-4)/2) + fm.getAscent());
  }

  public void drawShadow(Graphics g, int x, int y, int width, 
			 int height) {
    g.setColor(Color.darkGray);
    g.fillRect(x, y, width, height);
  }
}
