/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Fri Sep 15 21:54:48 2000 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package ix.test;

import java.util.*;

import ix.icon.*;

import ix.util.*;
import ix.util.lisp.*;


/**
 * A simple example using an I-X framework.
 */

public class PicoIXTest {

    static class PIX extends PicoIX { 	// just to have a shorter name
    }

    /**
     * Main program.
     */
    public static void main(String[] argv) {

	PIX.T_IX_Component example = new PIX.T_IX_Component();

	example.addIssueHandlers(makeTestIssueHandlers());
	example.setListener(new PIX.T_IX_ComponentListener());
	example.start();

	example.newEvent
	    (new PIX.T_Issue("twice", new PIX.T_Issue("return", "hello")));

	example.newEvent
	    (new PIX.T_Issue("stop when finished"));

    }

    /**
     * Create issue handlers as "instant" subclasses of T_IssueHandler.
     */
    static Object[] makeTestIssueHandlers() {
	return new Object[] {

	    // Post an issue twice.
	    new PIX.T_IssueHandler("twice") {
		void handleIssue(PIX.T_Issue i) {
		    postIssue((Issue)i.object);
		    postIssue((Issue)i.object);
		}
	    },

            // Send something to the listener.
	    new PIX.T_IssueHandler("return") {
		void handleIssue(PIX.T_Issue i) {
		    component.notifyListener(i.object);
		}
	    },

	    // Stop the component.
	    new PIX.T_IssueHandler("stop when finished") {
		void handleIssue(PIX.T_Issue i) {
		    if (component.controller.issues.isEmpty())
			component.stop();
		    else
			postIssue(i); 	// go to the end of the queue
		}
	    }
	};
    }

}
