/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Fri Nov 24 16:26:00 2000 by Jeff Dalton
 * Copyright: (c) 2000, AIAI, University of Edinburgh
 */

package ix.examples;

import java.util.*;

import ix.util.*;
import ix.util.lisp.*;


/**
 * Simple world state constraints for use with MicroISim.
 */

public class SimWorldState {

    /**
     * Constraints for a "pattern = value" world state.
     */
    static class Constraint extends PicoIX.Constraint {

	// An Object for matching these constraints with the CM.
	static final Symbol S_WORLD = Symbol.intern("world");

	// Constraint subtypes
	static final Symbol
	    S_CONDITION = Symbol.intern("condition"),
	    S_EFFECT    = Symbol.intern("effect");

	Object subtype;
	Object pattern;
	Object value;

	Constraint(Object subtype, Object pattern, Object value) {
	    super(S_WORLD);			// type
	    this.subtype = subtype;
	    this.pattern = pattern;
	    this.value = value;
	    this.args = Lisp.list(subtype, pattern, value); // for debugging
	}

    }


    /**
     * A constraint manager for a "pattern = value" world state.
     */
    static class CM extends PicoIX.ConstraintManager {

	Object type = Constraint.S_WORLD; 	// what we manage

	Hashtable stateTable = new Hashtable();

        CM() {
	    super(Constraint.S_WORLD); 	// type
	}

	Object addConstraint(PicoIX.Constraint c) {
	    Constraint wsc = (Constraint)c;
	    if (wsc.subtype == Constraint.S_CONDITION) {
		return addCondition(wsc);
	    }
	    else {
		Debug.assert(wsc.subtype == Constraint.S_EFFECT);
		return addEffect(wsc);
	    }
	}

	protected Object addCondition(Constraint cond) {
	    return tryCondition(cond);
	}

	protected Object addEffect(Constraint effect) {
	    stateTable.put(effect.pattern, effect.value);
	    return Boolean.TRUE;
	}

	Object tryConstraint(PicoIX.Constraint c) {
	    Constraint wsc = (Constraint)c;
	    if (wsc.subtype == Constraint.S_CONDITION) {
		return tryCondition(wsc);
	    }
	    else {
		Debug.assert(wsc.subtype == Constraint.S_EFFECT);
		return Boolean.TRUE;
	    }
	}

 	protected Object tryCondition(Constraint cond) {
 	    Object value = stateTable.get(cond.pattern);
 	    return new Boolean(value != null && value.equals(cond.value));
 	}

    }

}
